//
//  AsteroidObject.m
//  Quartzeroids2
//
//  Created by Matt Gallagher on 15/02/09.
//  Copyright 2009 Matt Gallagher. All rights reserved.
//
//  Permission is given to use this source code file without charge in any
//  project, commercial or otherwise, entirely at your risk, with the condition
//  that any redistribution (in part or whole) of source code must retain
//  this copyright and permission notice. Attribution in compiled projects is
//  appreciated but not required.
//

#import "AsteroidObject.h"
#import "GameData.h"

const double ASTEROID_LARGE_SIZE = 0.1;
const double ASTEROID_MAX_SPEED = 0.166;
const double ASTEROID_MEDIUM_SIZE = 0.065;
const double ASTEROID_MIN_SPEED = 0.033;
const double ASTEROID_SMALL_SIZE = 0.04;
const double ASTEROID_START_RADIUS = 0.40;
const NSInteger ASTEROID_BASE_NUM = 3;

@implementation AsteroidObject

//
// spawnNewAsteroidsReplacing:
//
// Creates new asteroids. If existingAsteroidKey is nil, level + 1 asteroids
// are spawned randomly placed in a ring around the player. If existingAsteroidKey
// is not nil, 3 asteroids of 1 size smaller than the existing are spawned at
// the existing's location and the existing is removed.
//
+ (void)spawnNewAsteroidsReplacing:(NSString *)existingAsteroidKey
{
	double x;
	double y;
	double size;
	
	if (existingAsteroidKey)
	{
		NSInteger destroyedCount =
			[[[[GameData sharedGameData] gameData]
				objectForKey:GAME_DATA_ASTEROIDS_DESTROYED_KEY] integerValue];
		destroyedCount += 1;
		[[GameData sharedGameData]
			setGameDataObject:[NSNumber numberWithInteger:destroyedCount]
			forKey:GAME_DATA_ASTEROIDS_DESTROYED_KEY];

		GameObject *existing = [[[GameData sharedGameData] gameObjects] objectForKey:existingAsteroidKey];
		size = existing.width;
		x = existing.x;
		y = existing.y;
		
		[[GameData sharedGameData] removeGameObjectForKey:existingAsteroidKey];
		
		if (size - DBL_EPSILON < ASTEROID_SMALL_SIZE)
		{
			NSInteger createdCount =
				[[[[GameData sharedGameData] gameData]
					objectForKey:GAME_DATA_ASTEROIDS_CREATED_KEY] integerValue];
			
			if (destroyedCount == createdCount)
			{
				[[GameData sharedGameData] newLevel];
				return;
			}

			return;
		}
		else if (size - DBL_EPSILON < ASTEROID_MEDIUM_SIZE)
		{
			size = ASTEROID_SMALL_SIZE;
		}
		else
		{
			size = ASTEROID_MEDIUM_SIZE;
		}
	}
	else
	{
		size = ASTEROID_LARGE_SIZE;
	}
	
	NSInteger level = [[[[GameData sharedGameData] gameData] objectForKey:GAME_DATA_LEVEL_KEY] integerValue];
	NSInteger i;
	for (i = 0; i < ASTEROID_BASE_NUM + (existingAsteroidKey ? 0 : (level - 1)); i++)
	{
		if (!existingAsteroidKey)
		{
			double angle = 2.0 * M_PI * (double)random() / (double)INT_MAX;
			x = 0.5 * GAME_ASPECT + ASTEROID_START_RADIUS * cos(angle);
			y = 0.5 + ASTEROID_START_RADIUS * sin(angle);
		}
		
		NSInteger newAsteroidIndex = [[[[GameData sharedGameData] gameData]
			objectForKey:GAME_DATA_ASTEROIDS_CREATED_KEY] integerValue];
		[[GameData sharedGameData]
			setGameDataObject:[NSNumber numberWithInteger:newAsteroidIndex + 1]
			forKey:GAME_DATA_ASTEROIDS_CREATED_KEY];

		AsteroidObject *asteroid = [[AsteroidObject alloc]
			initWithImageName:@"asteroid-back"
			x:x
			y:y
			width:size
			height:size
			visible:existingAsteroidKey ? YES : NO];
		[[GameData sharedGameData]
			addGameObject:asteroid
			forKey:[[GameData sharedGameData] keyForAsteroidAtIndex:newAsteroidIndex]];
		
		asteroid.trajectory = 2.0 * M_PI * (double)random() / (double)INT_MAX;
		asteroid.speed =
			ASTEROID_MIN_SPEED +
			(ASTEROID_MAX_SPEED - ASTEROID_MIN_SPEED) *
				(double)random() / (double)INT_MAX;
	}
}

@end
